package org.injac.cocoon.generators;

import java.io.IOException;
import java.net.URLDecoder;
import java.rmi.ServerException;
import java.util.*;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.generation.Generator;
import org.apache.cocoon.generation.ServiceableGenerator;
import org.apache.log4j.Logger;
import org.apache.webdav.lib.BaseProperty;
import org.apache.webdav.lib.methods.DepthSupport;
import org.apache.webdav.lib.methods.XMLResponseMethodBase;
import org.injac.utils.RequiredMetaData;
import org.injac.utils.ParseRequiredMetaData;
import org.injac.utils.WebdavConnection;
import org.injac.exceptions.*;

import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class MetadataGenerator extends ServiceableGenerator  implements Generator  {
	 /** The delay between checks on updates to the filesystem. */
    protected long refreshDelay;
    /** The URI of the namespace of this generator. */
    protected static final String URI = "http://esup-portail.org/injac";
    /** Root Node name */
    protected static final String ROOT_NODE_NAME = "response";
    /** The namespace prefix for this namespace. */
    protected static final String PREFIX = "MD";
    /** default meta-metadata name */
    protected static final String MDNAME = "INJAC:document-injac-metadata";
    protected static final String INJACTYPE = "INJAC:injac-type";
    
    protected String type, metadata;
	/**
	 * Static logger
	 */
	static Logger logger = Logger.getLogger(WebdavGenerator.class);

	/**
	 * Connection with session management
	 * 
	 * @uml.property name="webdavConnection"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	private WebdavConnection webdavConnection;

	/**
	 * name of the requested directory
	 */
	private String directory;
	/**
	 * The dav ressource uri
	 */
	private String webdavpath;
	/**
	 * Name of document in the path
	 */
	private String seldoc;
	/**
	 * depth of the harvest, by default 1
	 */
	private int depth=DepthSupport.DEPTH_1;
	
	/**
	 * name of meta-metadata
	 */
	private String mdname;
	/**
	 * Credentials
	 */
	private String login;
	private String password;
	
	/**
	 * Vector with requested MD name
	 */
	private Vector mdVector;
	
	/**
     * -Set the request parameters. Must be called before the generate method.
     * -Stock keys for caching.
     *
     * @param resolver     the SourceResolver object
     * @param objectModel  a <code>Map</code> containing model object
     * @param src          the directory to be XMLized specified as src attribute on &lt;map:generate/>
     * @param par          configuration parameters
     * 		first parama is injac-type name
     * 		second parama is document-injac-metadata name
     */
    public void setup(SourceResolver resolver, Map objectModel, String src, Parameters par)
            throws ProcessingException, SAXException, IOException {
        if (src == null) {
            throw new ProcessingException("No src attribute pointing to a Webdav resource whom metadatas to be XMLized specified.");
        }
        super.setup(resolver, objectModel, src, par);
        if(src.lastIndexOf('/')== src.length()-1)
			src = src.substring(0, src.length()-1);
        	
        logger.debug("directory : "+directory);
        seldoc = "";
        this.webdavpath = src;	
       
        webdavConnection = new WebdavConnection(objectModel);
        // 1. get parameters from pipeline
        mdVector = new Vector();
        String SD=par.getParameter("depth", "infinity");
		if(SD.equals("infinity"))
			depth= DepthSupport.DEPTH_INFINITY;
		else if(SD.equals("0"))
			depth= DepthSupport.DEPTH_0;
		else if(SD.equals("1"))
				depth= DepthSupport.DEPTH_1;
		
		mdname = par.getParameter("mdname", INJACTYPE+" "+ MDNAME);
		for(StringTokenizer ST = new StringTokenizer(mdname, " ");ST.hasMoreTokens(); )
        {
        	mdVector.add(ST.nextToken());
        }
		
		
		logger.debug("DEPTH : "+depth);
        //      Rcupration des credentials
		try {
			login=par.getParameter("login");
			password=par.getParameter("password");
		}
		catch(Exception e)
		{
			login=null;
			password=null;
		}
		
        // 2. Connect to webdavresource
        try {
			connect(webdavpath);
			logger.debug("MetadataGenerator::setup : connection ressource : "+webdavpath+ " okay");
        }catch(Exception e)
		{
        	logger.error("MetadataGenerator::setup : Can't establish webdav connection : "+webdavpath+"\nCheck either URI is incorrect or login/password error.\n"+e.getMessage());
        	
		}
        
        try {
			
			// test if it is a doc
			Vector vprops= new Vector();
			vprops.add("D:resourcetype");
			
			int index = webdavpath.indexOf("/slide/");
			// no slide context case
			if(index==-1)
				index = webdavpath.indexOf("/files");
			String rootpath = webdavpath.substring(index);
			Enumeration properties = webdavConnection.getProperties(rootpath, DepthSupport.DEPTH_0);
			HashMap map = new HashMap();
			while(properties.hasMoreElements()){
				BaseProperty prop = (BaseProperty)properties.nextElement();
				map.put(prop.getName(), URLDecoder.decode(prop.getPropertyAsString()));
				logger.debug("-----ENUM PROPS------->name :"+prop.getName()+"\tvalue : "+prop.getPropertyAsString());
			}
			String type=(String)map.get("injac-type");
			String rootdoc=(String)map.get("root-doc-name");
			if(type != null){
				if(type.equals("space")){
					if(rootdoc != null)
						seldoc=rootdoc;
					directory= src.substring(src.lastIndexOf('/')+1);
					logger.debug("MetadataGenerator::setup :\n\t path is of a space with root-doc-name="+ seldoc);
					
				}
				else
				{
					seldoc = webdavpath.substring(webdavpath.lastIndexOf('/')+1);
			    	webdavpath = webdavpath.substring(0, webdavpath.lastIndexOf('/'));
			    	directory= webdavpath.substring(webdavpath.lastIndexOf('/')+1);
			    	connect(webdavpath);
			    	logger.debug("MetadataGenerator::setup :\n\t path of a document("+ seldoc+")\nwebdavpath="+webdavpath);
				}
			}
			else {
				logger.error("Error accessing to "+rootpath+"\n"+map.toString());
				throw new ProcessingException("path is not of injac type or you don't have permission for this access"); 
			}
			logger.debug("MetadataGenerator::setup : connection ressource : "+webdavpath+ " okay");
        }catch(Exception e)
		{
        	logger.error("MetadataGenerator::setup : error toggling space/doc "+e.getClass()+e.getMessage()+"\nINFOS : webdavpath="+ webdavpath+"\tseldoc="+seldoc);
        	
		}
		
    }
	/* (non-Javadoc)
	 * @see org.apache.cocoon.generation.Generator#generate()
	 */
	public void generate() throws IOException, SAXException,
			ProcessingException {
		try {
			this.contentHandler.startDocument();
            this.contentHandler.startPrefixMapping(PREFIX, URI);
            AttributesImpl attributes = new AttributesImpl();
        	// attribut identifiant la resource
        	attributes.addAttribute("","directory", "directory", "CDATA", directory  );
        	attributes.addAttribute("","seldoc", "seldoc", "CDATA", seldoc  );
//        	dbut lment ressource
    		super.contentHandler.startElement(URI, ROOT_NODE_NAME, PREFIX + ':' + ROOT_NODE_NAME, attributes);
    		buildResponse();
//    		fin lment ressource
    		super.contentHandler.endElement(URI, ROOT_NODE_NAME, PREFIX + ':' + ROOT_NODE_NAME);
            
            this.contentHandler.endPrefixMapping(PREFIX);
            this.contentHandler.endDocument();
		}catch(Exception e)
		{
			StackTraceElement[] tab = e.getStackTrace();
			String ST="";
			for(int i=0;i<tab.length;i++)
			{
				ST+="\n"+tab[i].getFileName()+" l"+tab[i].getLineNumber()+" : "+ tab[i].getClassName()+"\t"+tab[i].getMethodName();
			
			}
			ST+="\n\nCause :"+e.getCause();
			logger.error("MetadataGenerator::generate : Soucis\n"+e.getMessage()+ST);
			
		}
		//webdavResource.closeSession();
		logger.debug("MetadataGenerator::generate : query on resource "+webdavpath+" ok.");
		
	}


    /**
	 * Connect to the WebDAV server	 
	 * @throws ServerException
	 */
	public void connect(String connexionString) throws Exception
	{
		logger.info("MetadataGenerator::connect try to connect webdavpath : "+webdavpath+"\nlogin : "+login);
		try 
		{
				if((login!=null) && (password!=null))
				{	
					webdavConnection.connect(connexionString, login, password);	
				}
				else
				{
					webdavConnection.connect(connexionString);	
				}
		}
		catch (Exception ex) 
		{	
			logger.error("error connecting to webdav resource : "+ connexionString+"\ncause : "+ex.getCause());	
	        throw ex;
	    }
		
		logger.info("MetadataGenerator::connect ---> OK");
	}
	void buildResponse()
	{
    	logger.info("MetadataGenerator::buildResponse");
		Enumeration resp_enum=null;
		Enumeration md=null;
		try
		{
			md = webdavConnection.getResponses(webdavpath, depth, mdVector);
			logger.info("MetadataGenerator::buildResponse propfindMethod ok");
		}catch(Exception e)
		{
			logger.error("MetadataGenerator::buildResponse : propfindMethod error:\npath: "+webdavpath+"\n"+e.getMessage());
			
		}
		// parsing responses
		try
		{
			int i=0;
			logger.debug("MetadataGenerator::buildResponse : PropFind methode ok");           
			while(md.hasMoreElements())
			{
				Object resp_element =md.nextElement();
				logger.debug("class resp_element "+i+" : "+resp_element.getClass());//+" resp_element.toString : "+resp_element.toString());
				XMLResponseMethodBase.Response resp= (XMLResponseMethodBase.Response)resp_element;
				addSpaceMD(resp);
				i++;
            }
			logger.info("MetadataGenerator::buildResponse parsing elements("+i+" elements) ok");
		}catch (Exception e) {
			logger.error("MetadataGenerator::buildResponse : parsing MD error\n webdavpath :"+webdavpath+"\n"+e.getMessage());
			
		}
	}
    /**
     * for a given space, add metadata in response document
     * @param resp
     */
private void addSpaceMD(XMLResponseMethodBase.Response resp){
	logger.debug("MetadataGenerator::addSpaceMD");
	String nodeName="metadatas";
	AttributesImpl attributes = new AttributesImpl();
	String href="";
	String value="";
	String mapname="";
	String tagname="";
	if(!resp.getProperties().hasMoreElements())
	{
		return;
	}
	// check injac type
	parseProps(resp);
	
	if(type != null && type.equals("space") && metadata != null)
	{
		//	 1. attributes for space identification
		try {
			href= resp.getHref();
			//href = URLDecoder.decode(href, "UTF-8");
			value= href.substring(href.lastIndexOf("/")+1);
			attributes.addAttribute("","encname", "encname", "CDATA", value  );
			value= URLDecoder.decode(value, "UTF-8");
			attributes.addAttribute("","name", "name", "CDATA", value  );
			attributes.addAttribute("","href", "href", "CDATA", href  );
		} catch (Exception e) {
			logger.error("attributes.addAttribute(\"\",\"name\", \"name\", \"CDATA\", "+value+")\n"+e.getMessage());
		}
	
		// 2. adding element
		try{
			// 	Decode URL
			value = java.net.URLDecoder.decode(metadata, "UTF-8");
			logger.debug("adding "+mdname+" in metadata elementfor "+href);
			//	metadata space element start
			super.contentHandler.startElement(URI, nodeName, PREFIX + ':' + nodeName, attributes);
			
			parseMD(value);
			//super.contentHandler.characters(value.toCharArray(), 0, value.length());	
			//	space element end
			super.contentHandler.endElement(URI, nodeName, PREFIX + ':' + nodeName);
		
		}catch(Exception se)
		{
			logger.error("MetadataGenerator::addSpaceMD Erreur de parsing element : tagname : "+tagname+" mapname : "+mapname+value+"\n"+se.getMessage());
		}
	}
}
/**
 * parse response to set type and metadata attributes
 * @param resp
 */
protected void parseProps(XMLResponseMethodBase.Response resp){
	try
	{
			
		Enumeration properties =resp.getProperties();
		
		while(properties.hasMoreElements()){
			BaseProperty prop = (BaseProperty)properties.nextElement();
			Element el=prop.getElement();
			String tagname =el.getNodeName();
			
			String value="";
			value = prop.getPropertyAsString();
			
			if(el.hasChildNodes())
				value=el.getFirstChild().getNodeValue();
			String mapname=tagname.substring(tagname.indexOf(":")+1);
			// test of injac-type
			if(mapname.equals(((String)mdVector.elementAt(0)).substring(((String)mdVector.elementAt(0)).indexOf(":")+1))){
				type=value;	
			}
			else if(mapname.equals(((String)mdVector.elementAt(1)).substring(((String)mdVector.elementAt(1)).indexOf(":")+1))){
				metadata = value;
			}
		}
	}catch(Exception e){
		
	}
}

protected void parseMD(String metas){
	String nodeName = "metadata";
	if(metas!=null){
		try{
			ParseRequiredMetaData parser = new ParseRequiredMetaData(metas);
		
			java.util.ArrayList required = parser.getMetadataListEdition();
			AttributesImpl attributes = new AttributesImpl();
			for(int i=0; i<required.size();i++){
				RequiredMetaData reqMD = (RequiredMetaData)required.get(i);
				String name =reqMD.getName();
			
				if(name!=null){
					super.contentHandler.startElement(URI, nodeName, PREFIX + ':' + nodeName, attributes);
					super.contentHandler.characters(name.toCharArray(), 0, name.length());	
					//	space element end
					super.contentHandler.endElement(URI, nodeName, PREFIX + ':' + nodeName);
				
					
				}
			}
		}catch(ExtractMetadataException ee){
			logger.error("unable to extract metadatas : "+ ee.getLocalizedMessage());
		}catch(SAXException se){
			logger.error("unable to write result document :"+se.getLocalizedMessage());
		}
	
	}
}
}
